# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base as base


class ImagesMemberRbacTest(base.BaseV2ImageRbacTest):

    credentials = ['primary', 'alt']

    @classmethod
    def resource_setup(cls):
        super(ImagesMemberRbacTest, cls).resource_setup()
        cls.tenant_id = cls.image_member_client.tenant_id
        cls.alt_tenant_id = cls.os_alt.image_member_client_v2.tenant_id

    @classmethod
    def setup_clients(cls):
        super(ImagesMemberRbacTest, cls).setup_clients()
        cls.image_client = cls.os_primary.image_client_v2
        cls.image_member_client = cls.os_primary.image_member_client_v2

    @rbac_rule_validation.action(service="glance",
                                 rules=["add_member"])
    @decorators.idempotent_id('b1b85ace-6484-11e6-881e-080027d0d606')
    def test_add_image_member(self):

        """Add image member

        RBAC test for the glance add_member policy
        """
        image_id = self.create_image()['id']
        # Toggle role and add image member
        with self.override_role():
            self.image_member_client.create_image_member(
                image_id, member=self.alt_tenant_id)

    @rbac_rule_validation.action(service="glance",
                                 rules=["delete_member"])
    @decorators.idempotent_id('ba075234-6484-11e6-881e-080027d0d606')
    def test_delete_image_member(self):

        """Delete image member

        RBAC test for the glance delete_member policy
        """
        image_id = self.create_image()['id']
        self.image_member_client.create_image_member(image_id,
                                                     member=self.alt_tenant_id)
        # Toggle role and delete image member
        with self.override_role():
            self.image_member_client.delete_image_member(image_id,
                                                         self.alt_tenant_id)

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_member"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('c01fd308-6484-11e6-881e-080027d0d606')
    def test_show_image_member(self):

        """Show image member

        RBAC test for the glance get_member policy
        """
        image_id = self.create_image()['id']
        self.image_member_client.create_image_member(
            image_id,
            member=self.alt_tenant_id)

        # Toggle role and get image member
        with self.override_role():
            self.image_member_client.show_image_member(image_id,
                                                       self.alt_tenant_id)

    @rbac_rule_validation.action(service="glance",
                                 rules=["modify_member"])
    @decorators.idempotent_id('ca448bb2-6484-11e6-881e-080027d0d606')
    def test_update_image_member(self):

        """Update image member

        RBAC test for the glance modify_member policy
        """
        image_id = self.create_image(visibility='shared')['id']
        self.image_member_client.create_image_member(
            image_id,
            member=self.tenant_id)
        self.image_member_client.update_image_member(
            image_id, self.tenant_id,
            status='accepted')
        # Toggle role and update member
        with self.override_role():
            self.image_member_client.update_image_member(
                image_id, self.tenant_id,
                status='pending')

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_members"])
    @decorators.idempotent_id('d0a2dc20-6484-11e6-881e-080027d0d606')
    def test_list_image_members(self):

        """List image member

        RBAC test for the glance get_members policy
        """
        image_id = self.create_image()['id']
        self.image_member_client.create_image_member(image_id,
                                                     member=self.alt_tenant_id)
        # Toggle role and list image members
        with self.override_role():
            self.image_member_client.list_image_members(image_id)



# -*- coding: utf-8 -*-
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('../../'))
sys.path.insert(0, os.path.abspath('../'))
sys.path.insert(0, os.path.abspath('./'))

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.todo',
    'sphinx.ext.viewcode',
    'openstackdocstheme',
    'oslo_config.sphinxconfiggen',
    'sphinxcontrib.apidoc',
]

# sphinxcontrib.apidoc options
apidoc_module_dir = '../../patrole_tempest_plugin'
apidoc_output_dir = 'framework/code'
apidoc_excluded_paths = [
    'hacking',
    'hacking/*',
    'tests',
    'tests/*',
    'config.py',
    'plugin.py',
    'version.py'
]
apidoc_separate_modules = True

config_generator_config_file = '../../etc/config-generator.patrole.conf'
sample_config_basename = '_static/patrole'

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.

copyright = u'2017, Patrole Developers'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
html_static_path = ['_static']
html_theme = 'openstackdocs'

# openstackdocstheme options
repository_name = 'openstack/patrole'
bug_project = 'patrole'
bug_tag = ''

# Must set this variable to include year, month, day, hours, and minutes.
html_last_updated_fmt = '%Y-%m-%d %H:%M'

# Output file base name for HTML help builder.
htmlhelp_basename = 'patroledoc'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index',
     'patrole.tex',
     u'patrole Documentation',
     u'Patrole Developers', 'manual'),
]

# Example configuration for intersphinx: refer to the Python standard library.
#intersphinx_mapping = {'http://docs.python.org/': None}



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


# TODO(gmann): Remove this test class once the nova queens branch goes
# into extended maintenance mode.
class FloatingIpsBulkRbacTest(rbac_base.BaseV2ComputeRbacTest):

    # Tests will fail with a 404 starting from microversion 2.36:
    # See the following link for details:
    # https://developer.openstack.org/api-ref/compute/#floating-ips-bulk-os-floating-ips-bulk-deprecated
    max_microversion = '2.35'
    depends_on_nova_network = True

    @classmethod
    def skip_checks(cls):
        super(FloatingIpsBulkRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-floating-ips-bulk', 'compute'):
            msg = "%s skipped as os-floating-ips-bulk extension not enabled." \
                  % cls.__name__
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def setup_clients(cls):
        super(FloatingIpsBulkRbacTest, cls).setup_clients()
        cls.fip_bulk_client = cls.os_primary.floating_ips_bulk_client

    @classmethod
    def resource_setup(cls):
        super(FloatingIpsBulkRbacTest, cls).resource_setup()
        cls.ip_range = CONF.validation.floating_ip_range
        cls.verify_unallocated_floating_ip_range(cls.ip_range)

    @classmethod
    def verify_unallocated_floating_ip_range(cls, ip_range):
        # Verify whether configure floating IP range is not already allocated.
        body = cls.fip_bulk_client.list_floating_ips_bulk()[
            'floating_ip_info']
        allocated_ips_list = map(lambda x: x['address'], body)
        for ip_addr in netaddr.IPNetwork(ip_range).iter_hosts():
            if str(ip_addr) in allocated_ips_list:
                msg = ("Configured unallocated floating IP range is already "
                       "allocated. Configure the correct unallocated range "
                       "as 'floating_ip_range'")
                raise lib_exc.InvalidConfiguration(msg)
        return

    def _create_floating_ips_bulk(self):
        pool = 'test_pool'
        # NOTE(felipemonteiro): Comment copied from Tempest. Reserving the IP
        # range but those are not attached anywhere. Using the below mentioned
        # interface which is not ever expected to be used. Clean up already
        # done for created IP range.
        interface = 'eth0'
        body = self.fip_bulk_client.create_floating_ips_bulk(
            self.ip_range, pool, interface)['floating_ips_bulk_create']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.fip_bulk_client.delete_floating_ips_bulk,
                        self.ip_range)
        return body

    @decorators.idempotent_id('9a49e73f-96a0-4e93-830a-22c4e443b486')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-floating-ips-bulk"])
    def test_create_floating_ips_bulk(self):
        with self.override_role():
            self._create_floating_ips_bulk()

    @decorators.idempotent_id('3b5c8a02-005d-4256-8a95-6fa2f389c6cf')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-floating-ips-bulk"])
    def test_list_floating_ips_bulk(self):
        with self.override_role():
            self.fip_bulk_client.list_floating_ips_bulk()

    @decorators.idempotent_id('37c2b759-c494-4e20-9dba-6a67b2df9573')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-floating-ips-bulk"])
    def test_delete_floating_ips_bulk(self):
        self._create_floating_ips_bulk()
        with self.override_role():
            self.fip_bulk_client.delete_floating_ips_bulk(self.ip_range)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class PolicyMinimumBandwidthRuleExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(PolicyMinimumBandwidthRuleExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('qos', 'network'):
            msg = "qos extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(PolicyMinimumBandwidthRuleExtRbacTest, cls).resource_setup()
        name = data_utils.rand_name(cls.__class__.__name__ + '-qos')
        cls.policy_id = cls.ntp_client.create_qos_policy(
            name=name)["policy"]["id"]
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.ntp_client.delete_qos_policy,
                                    cls.policy_id)

    def create_minimum_bandwidth_rule(self):
        rule = self.ntp_client.create_minimum_bandwidth_rule(
            self.policy_id, direction="egress", min_kbps=1000)
        rule_id = rule['minimum_bandwidth_rule']['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ntp_client.delete_minimum_bandwidth_rule,
                        self.policy_id, rule_id)
        return rule_id

    @decorators.idempotent_id('25B5EF3A-DF2A-4C80-A498-3BE14A321D97')
    @rbac_rule_validation.action(
        service="neutron", rules=["create_policy_minimum_bandwidth_rule"])
    def test_create_policy_minimum_bandwidth_rule(self):
        """Create policy_minimum_bandwidth_rule.

        RBAC test for the neutron "create_policy_minimum_bandwidth_rule" policy
        """

        with self.override_role():
            self.create_minimum_bandwidth_rule()

    @decorators.idempotent_id('01DD902C-47C5-45D2-9A0E-7AF05981DF21')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_minimum_bandwidth_rule"],
                                 expected_error_codes=[404])
    def test_show_policy_minimum_bandwidth_rule(self):
        """Show policy_minimum_bandwidth_rule.

        RBAC test for the neutron "get_policy_minimum_bandwidth_rule" policy
        """
        rule_id = self.create_minimum_bandwidth_rule()

        with self.override_role():
            self.ntp_client.show_minimum_bandwidth_rule(
                self.policy_id, rule_id)

    @decorators.idempotent_id('50AFE69B-455C-413A-BDC6-26B42DC8D55D')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_policy_minimum_bandwidth_rule",
               "update_policy_minimum_bandwidth_rule"],
        expected_error_codes=[404, 403])
    def test_update_policy_minimum_bandwidth_rule(self):
        """Update policy_minimum_bandwidth_rule.

        RBAC test for the neutron "update_policy_minimum_bandwidth_rule" policy
        """
        rule_id = self.create_minimum_bandwidth_rule()

        with self.override_role():
            self.ntp_client.update_minimum_bandwidth_rule(
                self.policy_id, rule_id, min_kbps=2000)

    @decorators.idempotent_id('2112E325-C3B2-4071-8A93-B218F275A83B')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_policy_minimum_bandwidth_rule",
               "delete_policy_minimum_bandwidth_rule"],
        expected_error_codes=[404, 403])
    def test_delete_policy_minimum_bandwidth_rule(self):
        """Delete policy_minimum_bandwidth_rule.

        RBAC test for the neutron "delete_policy_minimum_bandwidth_rule" policy
        """
        rule_id = self.create_minimum_bandwidth_rule()

        with self.override_role():
            self.ntp_client.delete_minimum_bandwidth_rule(
                self.policy_id, rule_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import compute
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumesActionsV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def setup_clients(cls):
        super(VolumesActionsV3RbacTest, cls).setup_clients()
        cls.image_client = cls.os_primary.image_client_v2

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(VolumesActionsV3RbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(VolumesActionsV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()

    def _create_server(self):
        tenant_network = self.get_tenant_network()
        server, _ = compute.create_test_server(
            self.os_primary, tenant_network=tenant_network,
            wait_until='ACTIVE')
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.servers_client.delete_server, server['id'])
        return server

    def _attach_volume(self, server, volume_id=None):
        if volume_id is None:
            volume_id = self.volume['id']

        self.servers_client.attach_volume(
            server['id'], volumeId=volume_id,
            device='/dev/%s' % CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_id, 'in-use')
        self.addCleanup(self._detach_volume, volume_id)

    def _detach_volume(self, volume_id=None):
        if volume_id is None:
            volume_id = self.volume['id']

        self.volumes_client.detach_volume(volume_id)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_id, 'available')

    @testtools.skipUnless(
        CONF.policy_feature_enabled
        .volume_extension_volume_actions_attach_policy,
        '"volume_extension:volume_actions:attach" must be available in the '
        'cloud.')
    @utils.services('compute')
    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:attach"])
    @decorators.idempotent_id('f97b10e4-2eed-4f8b-8632-71c02cb9fe42')
    def test_attach_volume_to_instance(self):
        server = self._create_server()
        volume_id = self.volume['id']

        with self.override_role():
            self.servers_client.attach_volume(
                server['id'], volumeId=volume_id,
                device='/dev/%s' % CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_id, 'in-use')
        self.addCleanup(self._detach_volume, volume_id)

    @utils.services('compute')
    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:detach"])
    @decorators.idempotent_id('5a042f6a-688b-42e6-a02e-fe5c47b89b07')
    def test_detach_volume_from_instance(self):
        server = self._create_server()
        self._attach_volume(server)
        volume_id = self.volume['id']

        with self.override_role():
            self.volumes_client.detach_volume(volume_id)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_id, 'available')

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:update_readonly_flag"])
    @decorators.idempotent_id('2750717a-f250-4e41-9e09-02624aad6ff8')
    def test_volume_readonly_update(self):
        with self.override_role():
            self.volumes_client.update_volume_readonly(self.volume['id'],
                                                       readonly=True)
        self.addCleanup(self.volumes_client.update_volume_readonly,
                        self.volume['id'], readonly=False)

    @decorators.idempotent_id('59b783c0-f4ef-430c-8a90-1bad97d4ec5c')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:update"])
    def test_volume_set_bootable(self):
        with self.override_role():
            self.volumes_client.set_bootable_volume(self.volume['id'],
                                                    bootable=True)

    @testtools.skipUnless(
        CONF.policy_feature_enabled
        .volume_extension_volume_actions_reserve_policy,
        '"volume_extension:volume_actions:reserve" must be available in the '
        'cloud.')
    @decorators.idempotent_id('41566922-75a1-4484-99c7-9c8782ee99ac')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:reserve"])
    def test_volume_reserve(self):
        with self.override_role():
            self.volumes_client.reserve_volume(self.volume['id'])

    @testtools.skipUnless(
        CONF.policy_feature_enabled
        .volume_extension_volume_actions_unreserve_policy,
        '"volume_extension:volume_actions:unreserve" must be available in the '
        'cloud.')
    @decorators.idempotent_id('e5fa9564-77d9-4e57-b0c0-3e0ae4d08535')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:unreserve"])
    def test_volume_unreserve(self):
        with self.override_role():
            self.volumes_client.unreserve_volume(self.volume['id'])

    @decorators.idempotent_id('c015c82f-7010-48cc-bd71-4ef542046f20')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:retype"])
    def test_volume_retype(self):
        vol_type = self.create_volume_type()['name']
        volume = self.create_volume()

        with self.override_role():
            self.volumes_client.retype_volume(volume['id'], new_type=vol_type)
        waiters.wait_for_volume_retype(
            self.volumes_client, volume['id'], vol_type)

    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_admin_actions:reset_status"])
    @decorators.idempotent_id('4b3dad7d-0e73-4839-8781-796dd3d7af1d')
    def test_volume_reset_status(self):
        volume = self.create_volume()

        with self.override_role():
            self.volumes_client.reset_volume_status(
                volume['id'], status='error')

    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_admin_actions:force_delete"])
    @decorators.idempotent_id('a312a937-6abf-4b91-a950-747086cbce48')
    def test_volume_force_delete(self):
        volume = self.create_volume()
        self.volumes_client.reset_volume_status(volume['id'], status='error')

        with self.override_role():
            self.volumes_client.force_delete_volume(volume['id'])
        self.volumes_client.wait_for_resource_deletion(volume['id'])

    @decorators.idempotent_id('48bd302b-950a-4830-840c-3158246ecdcc')
    @utils.services('compute')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_admin_actions:force_detach"])
    def test_force_detach_volume_from_instance(self):
        volume = self.create_volume()
        server = self._create_server()
        self._attach_volume(server, volume['id'])
        attachment = self.volumes_client.show_volume(
            volume['id'])['volume']['attachments'][0]

        # Reset volume's status to error.
        self.volumes_client.reset_volume_status(volume['id'], status='error')

        with self.override_role():
            self.volumes_client.force_detach_volume(
                volume['id'], connector=None,
                attachment_id=attachment['attachment_id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')


class VolumesActionsV310RbacTest(rbac_base.BaseVolumeRbacTest):
    _api_version = 3
    min_microversion = '3.10'
    max_microversion = 'latest'

    @classmethod
    def setup_clients(cls):
        super(VolumesActionsV310RbacTest, cls).setup_clients()
        cls.image_client = cls.os_primary.image_client_v2

    @classmethod
    def resource_setup(cls):
        super(VolumesActionsV310RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()

    @decorators.attr(type=["slow"])
    @utils.services('image')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:upload_image"])
    @decorators.idempotent_id('b0d0da46-903c-4445-893e-20e680d68b50')
    def test_volume_upload_image(self):
        # TODO(felipemonteiro): The ``upload_volume`` endpoint also enforces
        # "volume:copy_volume_to_image".
        image_name = data_utils.rand_name(self.__class__.__name__ + '-Image')

        with self.override_role():
            body = self.volumes_client.upload_volume(
                self.volume['id'], image_name=image_name, visibility="private",
                disk_format=CONF.volume.disk_format)['os-volume_upload_image']
        image_id = body["image_id"]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.image_client.delete_image,
                        image_id)
        waiters.wait_for_image_status(self.image_client, image_id,
                                      'active')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

    @decorators.attr(type=["slow"])
    @utils.services('image')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_actions:upload_public"])
    @decorators.idempotent_id('578a84dd-a6bd-4f97-a418-4a0c3c272c08')
    def test_volume_upload_public(self):
        # This also enforces "volume_extension:volume_actions:upload_image".
        image_name = data_utils.rand_name(self.__class__.__name__ + '-Image')

        with self.override_role():
            body = self.volumes_client.upload_volume(
                self.volume['id'], image_name=image_name, visibility="public",
                disk_format=CONF.volume.disk_format)['os-volume_upload_image']
            image_id = body["image_id"]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.image_client.delete_image,
                        image_id)
        waiters.wait_for_image_status(self.image_client, image_id,
                                      'active')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')


class VolumesActionsV312RbacTest(rbac_base.BaseVolumeRbacTest):
    _api_version = 3
    min_microversion = '3.12'
    max_microversion = 'latest'

    @decorators.idempotent_id('a654833d-4811-4acd-93ef-5ac4a34c75bc')
    @rbac_rule_validation.action(service="cinder", rules=["volume:get_all"])
    def test_show_volume_summary(self):
        with self.override_role():
            self.volumes_client.show_volume_summary()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityPolicyAssociationRbacTest(
        rbac_base.BaseIdentityV3RbacTest):

    def setUp(self):
        super(IdentityPolicyAssociationRbacTest, self).setUp()
        self.policy_id = self.setup_test_policy()['id']
        self.service_id = self.setup_test_service()['id']
        self.endpoint_id = self.setup_test_endpoint()['id']
        self.region_id = self.setup_test_region()['id']

    def _update_policy_association_for_endpoint(self, policy_id, endpoint_id):
        self.policies_client.update_policy_association_for_endpoint(
            policy_id, endpoint_id)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.policies_client.delete_policy_association_for_endpoint,
            policy_id, endpoint_id)

    def _update_policy_association_for_service(self, policy_id, service_id):
        self.policies_client.update_policy_association_for_service(
            policy_id, service_id)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.policies_client.delete_policy_association_for_service,
            policy_id, service_id)

    def _update_policy_association_for_region_and_service(
            self, policy_id, service_id, region_id):
        self.policies_client.update_policy_association_for_region_and_service(
            policy_id, service_id, region_id)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.policies_client.
            delete_policy_association_for_region_and_service,
            policy_id, service_id, region_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:create_policy_association_for_endpoint"])
    @decorators.idempotent_id('1b3f4f62-4f4a-4d27-be27-9a113058597f')
    def test_update_policy_association_for_endpoint(self):
        with self.override_role():
            self._update_policy_association_for_endpoint(
                self.policy_id, self.endpoint_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:check_policy_association_for_endpoint"])
    @decorators.idempotent_id('25ce8c89-e751-465c-8d35-52bacd774beb')
    def test_show_policy_association_for_endpoint(self):
        self._update_policy_association_for_endpoint(
            self.policy_id, self.endpoint_id)
        with self.override_role():
            self.policies_client.show_policy_association_for_endpoint(
                self.policy_id, self.endpoint_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:delete_policy_association_for_endpoint"])
    @decorators.idempotent_id('95cad2d8-bcd0-4c4e-a8f7-cc80601e43a1')
    def test_delete_policy_association_for_endpoint(self):
        self._update_policy_association_for_endpoint(
            self.policy_id, self.endpoint_id)
        with self.override_role():
            self.policies_client.delete_policy_association_for_endpoint(
                self.policy_id, self.endpoint_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:create_policy_association_for_service"])
    @decorators.idempotent_id('57fb80fe-6ce2-4995-b710-4692b3fc3cdc')
    def test_update_policy_association_for_service(self):
        with self.override_role():
            self._update_policy_association_for_service(
                self.policy_id, self.service_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:check_policy_association_for_service"])
    @decorators.idempotent_id('5cbe285f-4888-4f98-978f-30210ff28b74')
    def test_show_policy_association_for_service(self):
        self._update_policy_association_for_service(
            self.policy_id, self.service_id)
        with self.override_role():
            self.policies_client.show_policy_association_for_service(
                self.policy_id, self.service_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:delete_policy_association_for_service"])
    @decorators.idempotent_id('f754455c-02a4-4fb6-8c73-64ef453f955f')
    def test_delete_policy_association_for_service(self):
        self._update_policy_association_for_service(
            self.policy_id, self.service_id)
        with self.override_role():
            self.policies_client.delete_policy_association_for_service(
                self.policy_id, self.service_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:create_policy_association_for_region_and_service"])
    @decorators.idempotent_id('54d2a93e-c84d-4079-8ea9-2fb227c262a1')
    def test_update_policy_association_for_region_and_service(self):
        with self.override_role():
            self._update_policy_association_for_region_and_service(
                self.policy_id, self.service_id, self.region_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:check_policy_association_for_region_and_service"])
    @decorators.idempotent_id('0763b780-52c1-47bc-9316-1fe12a2ab0bc')
    def test_show_policy_association_for_region_and_service(self):
        self._update_policy_association_for_region_and_service(
            self.policy_id, self.service_id, self.region_id)
        with self.override_role():
            self.policies_client\
                .show_policy_association_for_region_and_service(
                    self.policy_id, self.service_id, self.region_id)

    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:delete_policy_association_for_region_and_service"])
    @decorators.idempotent_id('9c956888-81d4-4a24-8203-bff7b8a7834c')
    def test_delete_policy_association_for_region_and_service(self):
        self._update_policy_association_for_region_and_service(
            self.policy_id, self.service_id, self.region_id)
        with self.override_role():
            self.policies_client.\
                delete_policy_association_for_region_and_service(
                    self.policy_id, self.service_id, self.region_id)



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class NetworkIpAvailabilityExtRbacTest(base.BaseNetworkExtRbacTest):
    @classmethod
    def skip_checks(cls):
        super(NetworkIpAvailabilityExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('network-ip-availability',
                                          'network'):
            msg = "network-ip-availability extension not enabled."
            raise cls.skipException(msg)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network_ip_availability"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('93edc5ed-385f-4a8e-9b15-4370ec608253')
    def test_get_network_ip_availability(self):
        """Get network availability

        RBAC test for the neutron get_network_ip_availability policy
        """

        network_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')
        network = self.create_network(network_name=network_name)

        with self.override_role():
            self.ntp_client.show_network_ip_availability(network['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network_ip_availability"])
    @decorators.idempotent_id('d4ceb5f0-2342-4412-a617-4e1aaf7fcaf0')
    def test_get_network_ip_availabilities(self):
        """List network ip availabilities

        RBAC test for the neutron "get_network_ip_availability" policy
        for the "list_network_ip_availabilities" action.
        """
        admin_resources = (self.ntp_client.list_network_ip_availabilities()
                           ["network_ip_availabilities"])
        with self.override_role_and_validate_list(
                admin_resources=admin_resources) as ctx:
            ctx.resources = (self.ntp_client.list_network_ip_availabilities()
                             ["network_ip_availabilities"])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumeHostsV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume_extension:hosts"])
    @decorators.idempotent_id('64e837f5-5452-4e26-b934-c721ea7a8644')
    def test_list_hosts(self):
        with self.override_role():
            self.volume_hosts_client.list_hosts()

    @decorators.idempotent_id('9ddf321e-788f-4787-b8cc-dfa59e264143')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume_extension:hosts"])
    def test_show_host(self):
        hosts = self.volume_hosts_client.list_hosts()['hosts']
        host_names = [host['host_name'] for host in hosts]
        self.assertNotEmpty(host_names, "No available volume host was found, "
                                        "all hosts found were: %s" % hosts)

        with self.override_role():
            self.volume_hosts_client.show_host(host_names[0])



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class FlavorAccessRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def resource_setup(cls):
        super(FlavorAccessRbacTest, cls).resource_setup()
        cls.flavor_id = cls.create_flavor(is_public=False)['id']
        cls.public_flavor_id = CONF.compute.flavor_ref
        cls.tenant_id = cls.os_primary.credentials.tenant_id

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('a2bd3740-765d-4c95-ac98-9e027378c75e')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-access"])
    def test_show_flavor_contains_is_public_key(self):
        public_flavor_id = CONF.compute.flavor_ref

        with self.override_role():
            body = self.flavors_client.show_flavor(public_flavor_id)[
                'flavor']

        expected_attr = 'os-flavor-access:is_public'
        if expected_attr not in body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('dd388146-9750-4124-82ba-62deff1052bb')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-access"])
    def test_list_flavors_details_contains_is_public_key(self):
        expected_attr = 'os-flavor-access:is_public'

        with self.override_role():
            flavors = self.flavors_client.list_flavors(detail=True)['flavors']
        # There should already be a public flavor available, namely
        # `CONF.compute.flavor_ref`.
        public_flavors = [f for f in flavors if expected_attr in f]

        # If the `expected_attr` was not found in any flavor, then policy
        # enforcement failed.
        if not public_flavors:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @decorators.idempotent_id('39cb5c8f-9990-436f-9282-fc76a41d9bac')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-access:add_tenant_access"])
    def test_add_flavor_access(self):
        with self.override_role():
            self.flavors_client.add_flavor_access(
                flavor_id=self.flavor_id, tenant_id=self.tenant_id)
        self.addCleanup(self.flavors_client.remove_flavor_access,
                        flavor_id=self.flavor_id, tenant_id=self.tenant_id)

    @decorators.idempotent_id('61b8621f-52e4-473a-8d07-e228af8853d1')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-access:remove_tenant_access"])
    def test_remove_flavor_access(self):
        self.flavors_client.add_flavor_access(
            flavor_id=self.flavor_id, tenant_id=self.tenant_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.flavors_client.remove_flavor_access,
                        flavor_id=self.flavor_id, tenant_id=self.tenant_id)

        with self.override_role():
            self.flavors_client.remove_flavor_access(
                flavor_id=self.flavor_id, tenant_id=self.tenant_id)

    @decorators.idempotent_id('e1cf59fb-7f32-40a1-96b9-248ab23dd581')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-access"])
    def test_list_flavor_access(self):
        # Add flavor access for os_primary so that it can access the flavor or
        # else a NotFound is raised.
        self.flavors_client.add_flavor_access(
            flavor_id=self.flavor_id, tenant_id=self.tenant_id)
        self.addCleanup(self.flavors_client.remove_flavor_access,
                        flavor_id=self.flavor_id, tenant_id=self.tenant_id)

        with self.override_role():
            self.flavors_client.list_flavor_access(self.flavor_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base as base

CONF = config.CONF


class MigrateServerV225RbacTest(base.BaseV2ComputeRbacTest):
    min_microversion = '2.25'
    max_microversion = 'latest'
    block_migration = 'auto'

    @classmethod
    def skip_checks(cls):
        super(MigrateServerV225RbacTest, cls).skip_checks()
        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping migration tests.")

    def _get_server_details(self, server_id):
        body = self.servers_client.show_server(server_id)['server']
        return body

    def _get_host_for_server(self, server_id):
        return self._get_server_details(server_id)['OS-EXT-SRV-ATTR:host']

    def _get_host_other_than(self, host):
        for target_host in self._get_compute_hostnames():
            if host != target_host:
                return target_host

    def _get_compute_hostnames(self):
        body = self.hosts_client.list_hosts()['hosts']
        return [
            host_record['host_name']
            for host_record in body
            if host_record['service'] == 'compute'
        ]

    @decorators.attr(type='slow')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration not available.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-migrate-server:migrate"])
    @decorators.idempotent_id('c6f1607c-9fed-4c00-807e-9ba675b98b1b')
    def test_cold_migration(self):
        server = self.create_test_server(wait_until="ACTIVE")
        with self.override_role():
            self.servers_client.migrate_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'VERIFY_RESIZE')

    @decorators.attr(type='slow')
    @testtools.skipUnless(CONF.compute_feature_enabled.live_migration,
                          'Live migration feature is not enabled.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-migrate-server:migrate_live"])
    @decorators.idempotent_id('33520834-72c8-4edb-a189-a2e0fc9eb0d3')
    def test_migration_live(self):
        server_id = self.create_test_server(wait_until="ACTIVE",
                                            volume_backed=False)['id']
        actual_host = self._get_host_for_server(server_id)
        target_host = self._get_host_other_than(actual_host)

        with self.override_role():
            self.servers_client.live_migrate_server(
                server_id, host=target_host,
                block_migration=self.block_migration)
        waiters.wait_for_server_status(self.servers_client,
                                       server_id, "ACTIVE")



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class DscpMarkingRuleExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(DscpMarkingRuleExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('qos', 'network'):
            msg = "qos extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(DscpMarkingRuleExtRbacTest, cls).resource_setup()
        name = data_utils.rand_name(cls.__class__.__name__ + '-qos')
        cls.policy_id = cls.ntp_client.create_qos_policy(
            name=name)["policy"]["id"]
        cls.addClassResourceCleanup(
            cls.ntp_client.delete_qos_policy, cls.policy_id)

    def create_policy_dscp_marking_rule(cls):
        rule = cls.ntp_client.create_dscp_marking_rule(cls.policy_id, 10)
        rule_id = rule['dscp_marking_rule']['id']
        cls.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.ntp_client.delete_dscp_marking_rule, cls.policy_id, rule_id)
        return rule_id

    @decorators.idempotent_id('2717AB75-E4CF-4CA4-AF04-5BEC0C808AA5')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_policy_dscp_marking_rule"])
    def test_create_policy_dscp_marking_rule(self):
        """Create policy_dscp_marking_rule.

        RBAC test for the neutron "create_policy_dscp_marking_rule" policy
        """

        with self.override_role():
            self.create_policy_dscp_marking_rule()

    @decorators.idempotent_id('3D68F50E-B948-4B25-8A72-F6F4890BBC6F')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_dscp_marking_rule"],
                                 expected_error_codes=[404])
    def test_show_policy_dscp_marking_rule(self):
        """Show policy_dscp_marking_rule.

        RBAC test for the neutron "get_policy_dscp_marking_rule" policy
        """
        rule_id = self.create_policy_dscp_marking_rule()

        with self.override_role():
            self.ntp_client.show_dscp_marking_rule(self.policy_id, rule_id)

    @decorators.idempotent_id('33830794-8731-45C3-BC97-17718555DD7C')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_dscp_marking_rule",
                                        "update_policy_dscp_marking_rule"],
                                 expected_error_codes=[404, 403])
    def test_update_policy_dscp_marking_rule(self):
        """Update policy_dscp_marking_rule.

        RBAC test for the neutron "update_policy_dscp_marking_rule" policy
        """
        rule_id = self.create_policy_dscp_marking_rule()

        with self.override_role():
            self.ntp_client.update_dscp_marking_rule(
                self.policy_id, rule_id, dscp_mark=16)

    @decorators.idempotent_id('7BF564DD-3648-4D12-8A8B-6D5E576D1843')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_dscp_marking_rule",
                                        "delete_policy_dscp_marking_rule"],
                                 expected_error_codes=[404, 403])
    def test_delete_policy_dscp_marking_rule(self):
        """Delete policy_dscp_marking_rule.

        RBAC test for the neutron "delete_policy_dscp_marking_rule" policy
        """
        rule_id = self.create_policy_dscp_marking_rule()

        with self.override_role():
            self.ntp_client.delete_dscp_marking_rule(self.policy_id, rule_id)

    @decorators.idempotent_id('c012fd4f-3a3e-4af4-9075-dd3e170daecd')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_dscp_marking_rule"])
    def test_list_policy_dscp_marking_rules(self):
        """List policy_dscp_marking_rules.

        RBAC test for the neutron ``list_dscp_marking_rules`` function and
        the ``get_policy_dscp_marking_rule`` policy
        """
        admin_resource_id = self.create_policy_dscp_marking_rule()
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.ntp_client.list_dscp_marking_rules(
                policy_id=self.policy_id)["dscp_marking_rules"]



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class QosExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(QosExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('qos', 'network'):
            msg = "qos extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(QosExtRbacTest, cls).resource_setup()
        cls.network = cls.create_network()

    def create_policy(self, name=None):
        name = name or data_utils.rand_name(self.__class__.__name__)
        policy = self.ntp_client.create_qos_policy(name)['policy']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ntp_client.delete_qos_policy, policy['id'])
        return policy

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_policy"],
                                 expected_error_codes=[403])
    @decorators.idempotent_id('2ade2e48-7f82-4650-a69c-933d8d594636')
    def test_create_policy(self):

        """Create Policy Test

        RBAC test for the neutron create_policy policy
        """
        with self.override_role():
            self.create_policy()

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('d004a8de-b226-4eb4-9fdc-8202a7f64c56')
    def test_get_policy(self):

        """Show Policy Test

        RBAC test for the neutron get_policy policy
        """
        policy = self.create_policy()
        with self.override_role():
            self.ntp_client.show_qos_policy(policy['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy", "update_policy"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('fb74d56f-1dfc-490b-a9e1-454af583eefb')
    def test_update_policy(self):

        """Update Policy Test

        RBAC test for the neutron update_policy policy
        """
        policy = self.create_policy()
        with self.override_role():
            self.ntp_client.update_qos_policy(policy['id'],
                                              description='updated')

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy", "delete_policy"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('ef4c23a6-4095-47a6-958e-1df585f7d8db')
    def test_delete_policy(self):

        """Delete Policy Test

        RBAC test for the neutron delete_policy policy
        """
        policy = self.create_policy()
        with self.override_role():
            self.ntp_client.delete_qos_policy(policy['id'])

    @rbac_rule_validation.action(service="neutron", rules=["get_policy"])
    @decorators.idempotent_id('e84cec88-8478-4787-b603-5fcdd8ed7bd5')
    def test_list_policies(self):
        """List Policies Test

        RBAC test for the neutron ``list_qos_policies`` function and
        the ``get_policy``
        """
        admin_resource_id = self.create_policy()['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.ntp_client.list_qos_policies(
                id=admin_resource_id)["policies"]



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class AddressScopeExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AddressScopeExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('address-scope', 'network'):
            msg = "address-scope extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(AddressScopeExtRbacTest, cls).resource_setup()
        cls.network = cls.create_network()

    def _create_address_scope(self, name=None, **kwargs):
        name = name or data_utils.rand_name(self.__class__.__name__)
        address_scope = self.ntp_client.create_address_scope(name=name,
                                                             ip_version=6,
                                                             **kwargs)
        address_scope = address_scope['address_scope']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ntp_client.delete_address_scope,
                        address_scope['id'])
        return address_scope

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_address_scope"],
                                 expected_error_codes=[403])
    @decorators.idempotent_id('8cb2d6b5-23c2-4648-997b-7a6ae55be3ad')
    def test_create_address_scope(self):

        """Create Address Scope

        RBAC test for the neutron create_address_scope policy
        """
        with self.override_role():
            self._create_address_scope()

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_address_scope",
                                        "create_address_scope:shared"],
                                 expected_error_codes=[403, 403])
    @decorators.idempotent_id('0c3f55c0-6ebe-4251-afca-62c5cb4632ca')
    def test_create_address_scope_shared(self):

        """Create Shared Address Scope

        RBAC test for the neutron create_address_scope:shared policy
        """
        with self.override_role():
            self._create_address_scope(shared=True)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_address_scope"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('a53f741b-46f6-412f-936f-ac920d449da8')
    def test_get_address_scope(self):

        """Get Address Scope

        RBAC test for the neutron get_address_scope policy
        """
        address_scope = self._create_address_scope()
        with self.override_role():
            self.ntp_client.show_address_scope(address_scope['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_address_scope",
                                        "update_address_scope"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('3ce4d606-e067-4ef5-840f-96c680226e73')
    def test_update_address_scope(self):

        """Update Address Scope

        RBAC test for neutron update_address_scope policy
        """
        address_scope = self._create_address_scope()
        name = data_utils.rand_name(self.__class__.__name__)
        with self.override_role():
            self.ntp_client.update_address_scope(address_scope['id'],
                                                 name=name)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_address_scope",
                                        "update_address_scope",
                                        "update_address_scope:shared"],
                                 expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('77d3a9d2-721a-4d9f-9654-6b52f113df85')
    def test_update_address_scope_shared(self):

        """Update Shared Address Scope

        RBAC test for neutron update_address_scope:shared policy
        """
        address_scope = self._create_address_scope(shared=True)
        with self.override_role():
            self.ntp_client.update_address_scope(address_scope['id'],
                                                 shared=True)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_address_scope",
                                        "delete_address_scope"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('277d8e47-e498-4452-b969-a91f747296ba')
    def test_delete_address_scope(self):

        """Delete Address Scope

        RBAC test for neutron delete_address_scope policy
        """
        address_scope = self._create_address_scope()
        with self.override_role():
            self.ntp_client.delete_address_scope(address_scope['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_address_scope"])
    @decorators.idempotent_id('c093fd34-96ee-4abe-8fa5-916dc29653e3')
    def test_list_address_scopes(self):
        """List Address Scopes

        RBAC test for the neutron ``list_address_scopes`` function and
        the ``get_address_scope`` policy
        """
        admin_resource_id = self._create_address_scope()['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.ntp_client.list_address_scopes(
                id=admin_resource_id)["address_scopes"]
